#include <stdio.h>
#include "common.h"
#include "fmopl.h"

// ------------------------- * Music * -------------------------
typedef	struct
{
	word length,
	values[1];
} MusicGroup;
MusicGroup *music=NULL;
word *sqHackPtr=NULL, sqHackLen=0;
long sqHackTime=0;
long alTimeCount=0;

static FM_OPL *hAdLibMus=NULL;

// ------------------------- * Devider * -------------------------

bool SD_MusInit(void)
{
	hAdLibMus=OPLCreate(OPL_TYPE_YM3812, 3579545, ADLIB_MUS_SPEED);
	if(hAdLibMus==NULL)
	{
		return false;
	}
	else
	{
		OPLWrite(hAdLibMus, 0x01, 0x20); /* Set WSE=1 */
		OPLWrite(hAdLibMus, 0x08, 0x00); /* Set CSM=0 & SEL=0 */
		return true;
	}
}

void SD_ShutDown(void)
{
	if(hAdLibMus)
	{
		OPLDestroy(hAdLibMus);
		hAdLibMus=NULL;
	}
}


// returns length in ms
void SD_LoadMusic(void *musbuffer)
{
	music=(MusicGroup*)musbuffer;
	sqHackPtr=music->values;
	sqHackLen=music->length;
	sqHackTime=alTimeCount=0;
}

// returns length in ms
int SD_GetLen(void *musbuffer)
{
	word *Ptr, Len;
	long Time;
	long alTime;

	Ptr=((MusicGroup*)musbuffer)->values;
	Len=((MusicGroup*)musbuffer)->length;
	Time=alTime=0;

// ---------------- AdLib Cycle start -------------------
	while(1)
	{
		while(Len && Time<=alTime)
		{
			Ptr++;
			Time=alTime+*Ptr++;
			Len-=4;
		}
		alTime++;
		if(Len<=0) break;
	}
// ---------------- AdLib Cycle end ---------------------
	return alTime*1000/700; // in milliseconds
}

// we'll do 44100/16/mono

// this is a callback from my snd_lib
// exactly [size] bytes must be written to buffer!
// I'm setting this to 1 second! So we must make 700 AdLib cycles here
int UpdateMusic(int pos, int size, void *buffer)
{
	byte *al;				//[2] {a, v} (register, value)
	int n;
	short *ptr;			// where to put data!
	int AdLibTicks;
	bool flag=false;

	if(sqHackLen<=0) return 0; // nothing to play

	AdLibTicks=700*size/ADLIB_MUS_BYPS; // how many AdLib ticks

	ptr=(short *)buffer;

	for(n=0; n<AdLibTicks; n++) //exactly one second
	{
// ---------------- AdLib Cycle start -------------------
		while(sqHackLen && (sqHackTime<=alTimeCount))
		{
			al=(byte *)sqHackPtr++;
			sqHackTime=alTimeCount+*sqHackPtr++;
			OPLWrite(hAdLibMus, al[0], al[1]);
			sqHackLen-=4;
		}
		alTimeCount++;
// ---------------- AdLib Cycle end ---------------------
	// now we'll get AdLib Output!
		YM3812UpdateOne(hAdLibMus, ptr, 63);
		ptr+=63;

		if(sqHackLen<=0) return (long)ptr-(long)buffer;
	}

	return AdLibTicks*ADLIB_MUS_BYPS/700;
}

// pos is in bytes
int Seek(int pos)
{
	byte *al;				//[2] {a, v} (register, value)
//	short ptr[63];			// where to put data!
	int AdLibTicks;
	bool flag=false;
	int spos;

	if(sqHackLen<=0) return -1; // nothing to seek

	AdLibTicks=700*(pos/ADLIB_MUS_BYPS); // how many AdLib ticks

	if(alTimeCount>AdLibTicks)
	{
		OPLResetChip(hAdLibMus);
		sqHackPtr=music->values;
		sqHackLen=music->length;
		sqHackTime=alTimeCount=0;
	}
	spos=alTimeCount*63*2; // old pos

	while(alTimeCount<AdLibTicks) //exactly one second
	{
// ---------------- AdLib Cycle start -------------------
		while(sqHackLen && (sqHackTime<=alTimeCount))
		{
			al=(byte *)sqHackPtr++;
			sqHackTime=alTimeCount+*sqHackPtr++;
			OPLWrite(hAdLibMus, al[0], al[1]);
			sqHackLen-=4;
		}
		alTimeCount++;

// ---------------- AdLib Cycle end ---------------------
	// now we'll get AdLib Output!
		//YM3812UpdateOne(hAdLibMus, ptr, 63);
		spos+=63*2;

		if(sqHackLen<=0) break; // seek past end
	}

	return spos;
}
// ------------------------- * End Music * -------------------------